//
//  ActivityRecognitionManager.swift
//  activity_rec_flutter
//
//  Created by WOO JIN HWANG on 2021/04/23.
//

import CoreMotion
import Foundation
import SQLite3

typealias ActivityDataHandler = (ActivityData) -> Void

class ActivityRecognitionManager {

  private let motionActivityManager = CMMotionActivityManager()
  private var isRunningService = false

  // Create an instance of UserDefaultsManager
  let userDefaultsManager = UserDefaultsManager.shared

  public func startService(handler: @escaping ActivityDataHandler) {
    if (isRunningService) {
      NSLog("ActivityRecognitionManager The activity recognition service has already started.")
      stopService()
    }

    motionActivityManager.startActivityUpdates(to: .main) { (activity) in
      guard let activity = activity else { return }

      let actData = ActivityData(from: activity)
      //let currentTime = String(Date().timeIntervalSince1970 * 1000)

      //todo write point calculation logic here
      let recordedActivity = userDefaultsManager.recordedActivity
      let recordedActivityTime = userDefaultsManager.recordedActivityTime

       print("ActivityRecognitionManager activity type: \(actData.type), confidence: \(actData.confidence), startDate: \(actData.startDate)")
       print("ActivityRecognitionManager userDefaultsManager recordedActivity =====> \(recordedActivity)")
       print("ActivityRecognitionManager userDefaultsManager recordedActivityTime =====> \(recordedActivityTime)")

       if(recordedActivity == ""){
        print("ActivityRecognitionManager =====> recordedActivity == null")
        userDefaultsManager.recordedActivity = actData.type
        userDefaultsManager.recordedActivityTime = actData.startDate

       }else{
       print("ActivityRecognitionManager =====> recordedActivity != null")
        if (actData.type != recordedActivity) {
           print("ActivityRecognitionManager =====> activityData != recordedActivity")
           let timeDifferenceMillis = Date().timeIntervalSince1970 - TimeInterval(recordedActivityTime)
           // Convert seconds to minutes
           let timeDifferenceMinutes = Int(timeDifferenceMillis / 60)
           userDefaultsManager.recordedActivity = actData.type
           userDefaultsManager.recordedActivityTime = Int64(Date().timeIntervalSince1970)

           // Assuming activityData is an object of a class or struct containing properties like 'type' and 'timeDifferenceMinutes'

           if let activityType = actData.type {
               switch activityType {
               case "IN_VEHICLE":
                   let points = Int(timeDifferenceMinutes) * 10 // per minute 10 points for vehicle
                   let storedPoints = userDefaultsManager.currentInVehiclePoints
                   let currentPoints = storedPoints
                   userDefaultsManager.currentInVehiclePoints = currentPoints + points

               case "ON_BICYCLE":
                   let points = Int(timeDifferenceMinutes) * 8 // per minute 8 points for bicycle
                   let storedPoints = userDefaultsManager.currentOnCyclePoints
                   let currentPoints = storedPoints
                   userDefaultsManager.currentOnCyclePoints = currentPoints + points

               case "RUNNING":
                   let points = Int(timeDifferenceMinutes) * 6 // per minute 6 points for running
                   let storedPoints = userDefaultsManager.currentRunningPoints
                   let currentPoints = storedPoints
                   userDefaultsManager.currentRunningPoints = currentPoints + points

               case "WALKING":
                   let points = Int(timeDifferenceMinutes) * 4 // per minute 4 points for walking
                   let storedPoints = userDefaultsManager.currentWalkingPoints
                   let currentPoints = storedPoints
                   userDefaultsManager.currentWalkingPoints = currentPoints + points

               case "STILL":
                   let points = Int(timeDifferenceMinutes) * 2 // per minute 2 points for still
                   let storedPoints = userDefaultsManager.currentStillPoints
                   let currentPoints = storedPoints
                   userDefaultsManager.currentStillPoints = currentPoints + points

               default:
                   print("UNKNOWN activity")
               }
           }

        }
       }

      DatabaseManager.shared.insertActivityData(type: actData.type, confidence: actData.confidence,startDate:actData.startDate)

      handler(ActivityData(from: activity))
    }
    isRunningService = true
  }
  
  public func stopService() {
    motionActivityManager.stopActivityUpdates()
    isRunningService = false
  }
}
